@@ -18,7 +18,8 @@ class PamWorker : public QObject
     Q_OBJECT
 public:
     PamWorker();
-    ~PamWorker();
+    ~PamWorker() override;
+    Q_DISABLE_COPY_MOVE(PamWorker)
     void start(const QString &service, const QString &user);
     void authenticate();
 
@@ -30,6 +31,8 @@ Q_SIGNALS:
     void errorMessage(const QString &msg);
     void failed();
     void succeeded();
+    void unavailabilityChanged(bool unavailable);
+    void inAuthenticateChanged(bool inAuthenticate);
 
     // internal
     void promptResponseReceived(const QByteArray &prompt);
@@ -41,8 +44,10 @@ private:
     pam_handle_t *m_handle = nullptr; //< the actual PAM handle
     struct pam_conv m_conv;
 
+    bool m_unavailable = false;
     bool m_inAuthenticate = false;
-    int m_result;
+    int m_result = -1;
+    QString m_service;
 };
 
 int PamWorker::converse(int n, const struct pam_message **msg, struct pam_response **resp, void *data)
@@ -71,18 +76,28 @@ int PamWorker::converse(int n, const struct pam_message **msg, struct pam_respon
                 Q_EMIT c->prompt(prompt);
             }
 
+            qCDebug(KSCREENLOCKER_GREET,
+                    "[PAM worker %s] Message: %s: %s",
+                    qUtf8Printable(c->m_service),
+                    (isSecret ? "Echo-off prompt" : "Echo-on prompt"),
+                    qUtf8Printable(prompt));
+
             QByteArray response;
             QEventLoop e;
             QObject::connect(c, &PamWorker::promptResponseReceived, &e, [&](const QByteArray &_response) {
+                qCDebug(KSCREENLOCKER_GREET, "[PAM worker %s] Received response, exiting nested event loop", qUtf8Printable(c->m_service));
                 response = _response;
                 e.exit(0);
             });
             QObject::connect(c, &PamWorker::cancelled, &e, [&]() {
+                qCDebug(KSCREENLOCKER_GREET, "[PAM worker %s] Received cancellation, exiting with PAM_CONV_ERR", qUtf8Printable(c->m_service));
                 e.exit(PAM_CONV_ERR);
             });
 
+            qCDebug(KSCREENLOCKER_GREET, "[PAM worker %s] Starting nested event loop to await response", qUtf8Printable(c->m_service));
             int rc = e.exec();
             if (rc != 0) {
+                qCDebug(KSCREENLOCKER_GREET, "[PAM worker %s] Nested event loop's exit code was not zero, bailing", qUtf8Printable(c->m_service));
                 return rc;
             }
 
@@ -105,15 +120,21 @@ int PamWorker::converse(int n, const struct pam_message **msg, struct pam_respon
 
             break;
         }
-        case PAM_ERROR_MSG:
-            qCDebug(KSCREENLOCKER_GREET) << QString::fromLocal8Bit(msg[i]->msg);
-            Q_EMIT c->errorMessage(QString::fromLocal8Bit(msg[i]->msg));
+        case PAM_ERROR_MSG: {
+            const QString error = QString::fromLocal8Bit(msg[i]->msg);
+            qCDebug(KSCREENLOCKER_GREET, "[PAM worker %s] Message: Error message: %s", qUtf8Printable(c->m_service), qUtf8Printable(error));
+            Q_EMIT c->errorMessage(error);
             break;
-        case PAM_TEXT_INFO:
+        }
+        case PAM_TEXT_INFO: {
             // if there's only the info message, let's predict the prompts too
-            qCDebug(KSCREENLOCKER_GREET) << QString::fromLocal8Bit(msg[i]->msg);
-            Q_EMIT c->infoMessage(QString::fromLocal8Bit(msg[i]->msg));
+            const QString info = QString::fromLocal8Bit(msg[i]->msg);
+            qCDebug(KSCREENLOCKER_GREET, "[PAM worker %s] Message: Info message: %s", qUtf8Printable(c->m_service), qUtf8Printable(info));
+            Q_EMIT c->infoMessage(info);
+            break;
+        }
         default:
+            qCDebug(KSCREENLOCKER_GREET, "[PAM worker %s] Message: Unhandled message type: %d", qUtf8Printable(c->m_service), msg[i]->msg_style);
             break;
         }
     }
@@ -123,8 +144,8 @@ int PamWorker::converse(int n, const struct pam_message **msg, struct pam_respon
 
 PamWorker::PamWorker()
     : QObject(nullptr)
+    , m_conv({&PamWorker::converse, this})
 {
-    m_conv = {&PamWorker::converse, this};
 }
 
 PamWorker::~PamWorker()
@@ -136,13 +157,18 @@ PamWorker::~PamWorker()
 
 void PamWorker::authenticate()
 {
-    if (m_inAuthenticate) {
+    if (m_inAuthenticate || m_unavailable) {
         return;
     }
     m_inAuthenticate = true;
-    qCDebug(KSCREENLOCKER_GREET) << "Start auth";
+    Q_EMIT inAuthenticateChanged(m_inAuthenticate);
+    qCDebug(KSCREENLOCKER_GREET, "[PAM worker %s] Authenticate: Starting authentication", qUtf8Printable(m_service));
     int rc = pam_authenticate(m_handle, 0); // PAM_SILENT);
-    qCDebug(KSCREENLOCKER_GREET) << "Auth done RC" << rc;
+    qCDebug(KSCREENLOCKER_GREET,
+            "[PAM worker %s] Authenticate: Authentication done, result code: %d (%s)",
+            qUtf8Printable(m_service),
+            rc,
+            pam_strerror(m_handle, rc));
 
     Q_EMIT busyChanged(false);
 
@@ -150,10 +176,14 @@ void PamWorker::authenticate()
         rc = pam_setcred(m_handle, PAM_REFRESH_CRED);
         /* ignore errors on refresh credentials. If this did not work we use the old ones. */
         Q_EMIT succeeded();
+    } else if (rc == PAM_AUTHINFO_UNAVAIL || rc == PAM_MODULE_UNKNOWN) {
+        m_unavailable = true;
+        Q_EMIT unavailabilityChanged(m_unavailable);
     } else {
         Q_EMIT failed();
     }
     m_inAuthenticate = false;
+    Q_EMIT inAuthenticateChanged(m_inAuthenticate);
 }
 
 static void fail_delay(int retval, unsigned usec_delay, void *appdata_ptr)
@@ -165,6 +195,7 @@ static void fail_delay(int retval, unsigned usec_delay, void *appdata_ptr)
 
 void PamWorker::start(const QString &service, const QString &user)
 {
+    m_service = service;
     if (user.isEmpty())
         m_result = pam_start(qPrintable(service), nullptr, &m_conv, &m_handle);
     else
@@ -176,14 +207,18 @@ void PamWorker::start(const QString &service, const QString &user)
 #endif
 
     if (m_result != PAM_SUCCESS) {
-        qCWarning(KSCREENLOCKER_GREET) << "[PAM] start" << pam_strerror(m_handle, m_result);
+        qCWarning(KSCREENLOCKER_GREET,
+                  "[PAM worker %s] start: error starting, result code: %d (%s)",
+                  qUtf8Printable(m_service),
+                  m_result,
+                  pam_strerror(m_handle, m_result));
         return;
     } else {
-        qCDebug(KSCREENLOCKER_GREET) << "[PAM] Starting...";
+        qCDebug(KSCREENLOCKER_GREET, "[PAM worker %s] start: successfully started", qUtf8Printable(m_service));
     }
 }
 
-PamAuthenticator::PamAuthenticator(const QString &service, const QString &user, QObject *parent)
+PamAuthenticator::PamAuthenticator(const QString &service, const QString &user, NoninteractiveAuthenticatorTypes types, QObject *parent)
     : QObject(parent)
     , m_signalsToMembers({
           {QMetaMethod::fromSignal(&PamAuthenticator::prompt), m_prompt},
@@ -191,9 +226,10 @@ PamAuthenticator::PamAuthenticator(const QString &service, const QString &user,
           {QMetaMethod::fromSignal(&PamAuthenticator::infoMessage), m_infoMessage},
           {QMetaMethod::fromSignal(&PamAuthenticator::errorMessage), m_errorMessage},
       })
+    , m_service(service)
+    , m_authenticatorType(types)
+    , d(new PamWorker)
 {
-    d = new PamWorker;
-
     d->moveToThread(&m_thread);
 
     connect(&m_thread, &QThread::finished, d, &QObject::deleteLater);
@@ -216,6 +252,15 @@ PamAuthenticator::PamAuthenticator(const QString &service, const QString &user,
         Q_EMIT errorMessage(msg);
     });
 
+    connect(d, &PamWorker::inAuthenticateChanged, this, [this](bool isInAuthenticate) {
+        m_inAuthentication = isInAuthenticate;
+        Q_EMIT availableChanged();
+    });
+    connect(d, &PamWorker::unavailabilityChanged, this, [this](bool isUnavailable) {
+        m_unavailable = isUnavailable;
+        Q_EMIT availableChanged();
+    });
+
     connect(d, &PamWorker::succeeded, this, [this]() {
         m_unlocked = true;
         Q_EMIT succeeded();
@@ -247,6 +292,16 @@ bool PamAuthenticator::isBusy() const
     return m_busy;
 }
 
+bool PamAuthenticator::isAvailable() const
+{
+    return m_inAuthentication && !m_unavailable;
+}
+
+PamAuthenticator::NoninteractiveAuthenticatorTypes PamAuthenticator::authenticatorType() const
+{
+    return m_authenticatorType;
+}
+
 void PamAuthenticator::setBusy(bool busy)
 {
     if (m_busy != busy) {
@@ -285,34 +340,6 @@ void PamAuthenticator::cancel()
     QMetaObject::invokeMethod(d, &PamWorker::cancelled);
 }
 
-// Force emit the signals when a view connects to them. This prevents a race condition where screens appear after
-// stateful signals (such as prompt) had been emitted and end up in an incorrect state.
-// (e.g. https://bugs.kde.org/show_bug.cgi?id=456210 where the view ends up in a no-prompt state)
-void PamAuthenticator::connectNotify(const QMetaMethod &signal)
-{
-    // TODO remove this function for Plasma 6. The properties should be bound to  so we don't need to force
-    // emit them every time a view connects.
-
-    // NOTE signals are queued because during connect qml is not yet ready to receive them
-
-    if (m_unlocked && signal == QMetaMethod::fromSignal(&PamAuthenticator::succeeded)) {
-        signal.invoke(this, Qt::QueuedConnection);
-        return;
-    }
-
-    for (const auto &[signalMethod, memberString] : m_signalsToMembers)
-    {
-        if (signal != signalMethod) {
-            continue;
-        }
-
-        if (!memberString.isNull()) {
-            signalMethod.invoke(this, Qt::QueuedConnection, Q_ARG(QString, memberString));
-            return;
-        }
-    }
-}
-
 QString PamAuthenticator::getPrompt() const
 {
     return m_prompt;
@@ -333,4 +360,11 @@ QString PamAuthenticator::getErrorMessage() const
     return m_errorMessage;
 }
 
+QString PamAuthenticator::service() const
+{
+    return m_service;
+}
+
 #include "pamauthenticator.moc"
+
+#include "moc_pamauthenticator.cpp"
